package gregapi.recipes;

import gregapi.config.ConfigCategories;
import gregapi.data.CS.ConfigsGT;
import gregapi.data.OP;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictManager;
import gregapi.oredict.OreDictMaterial;
import gregapi.oredict.OreDictMaterialStack;
import gregapi.util.OM;
import gregapi.util.UT;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;

/**
 * Class for Automatic Recipe registering.
 */
public class GT_RecipeRegistrator {
	public static void registerMaterialRecycling(ItemStack aStack, OreDictMaterial aMaterial, long aMaterialAmount, OreDictMaterialStack aByproduct) {
		if (UT.Stacks.invalid(aStack)) return;
		if (aByproduct != null) {aByproduct = aByproduct.clone(); aByproduct.mAmount /= aStack.stackSize;}
		OM.data(UT.Stacks.copyAmount(1, aStack), new OreDictItemData(aMaterial, aMaterialAmount / aStack.stackSize, aByproduct));
	}
	
	public static void registerMaterialRecycling(ItemStack aStack, OreDictItemData aData) {
		if (UT.Stacks.invalid(aStack) || UT.Stacks.equal(new ItemStack(Items.blaze_rod), aStack) || aData == null || !aData.hasValidMaterialData() || aData.mMaterial.mAmount <= 0 || UT.Fluids.getFluidForFilledItem(aStack, false) != null) return;
		registerReverseMacerating			(UT.Stacks.copyAmount(1, aStack), aData, aData.mPrefix == null);
		registerReverseSmelting				(UT.Stacks.copyAmount(1, aStack), aData.mMaterial.mMaterial, aData.mMaterial.mAmount, true);
		registerReverseFluidSmelting		(UT.Stacks.copyAmount(1, aStack), aData.mMaterial.mMaterial, aData.mMaterial.mAmount, aData.getByProduct(0));
		registerReverseArcSmelting			(UT.Stacks.copyAmount(1, aStack), aData);
	}
	
	/**
	 * @param aStack the stack to be recycled.
	 * @param aMaterial the Material.
	 * @param aMaterialAmount the amount of it in Material Units.
	 */
	public static void registerReverseFluidSmelting(ItemStack aStack, OreDictMaterial aMaterial, long aMaterialAmount, OreDictMaterialStack aByproduct) {
//		if (aStack == null || aMaterial == null || aMaterial.mSmeltInto.mStandardMoltenFluid == null || !aMaterial.contains(TD.Properties.SMELTING_TO_FLUID) || (L * aMaterialAmount) / (U * aStack.stackSize) <= 0) return;
//		RA.addFluidSmelterRecipe(UT.Stacks.copyAmount(1, aStack), aByproduct==null?null:aByproduct.mMaterial.contains(TD.Properties.NO_SMELTING)||!aByproduct.mMaterial.contains(TD.Atomic.METAL)?aByproduct.mMaterial.contains(TD.Properties.FLAMMABLE)?OM.dust(MT.Ash, aByproduct.mAmount/2):aByproduct.mMaterial.contains(TD.Properties.UNBURNABLE)?OM.dustOrIngot(aByproduct.mMaterial.mSmeltInto, aByproduct.mAmount):null:OM.ingotOrDust(aByproduct.mMaterial.mSmeltInto, aByproduct.mAmount), aMaterial.mSmeltInto.getMolten((L * aMaterialAmount) / (U * aStack.stackSize)), 10000, (int)Math.max(1, (24 * aMaterialAmount) / U), Math.max(8, (int)Math.sqrt(2*aMaterial.mSmeltInto.mStandardMoltenFluid.getTemperature())));
	}
	
	/**
	 * @param aStack the stack to be recycled.
	 * @param aMaterial the Material.
	 * @param aMaterialAmount the amount of it in Material Units.
	 * @param aAllowAlloySmelter if it is allowed to be recycled inside the Alloy Smelter.
	 */
	public static void registerReverseSmelting(ItemStack aStack, OreDictMaterial aMaterial, long aMaterialAmount, boolean aAllowAlloySmelter) {
//		if (aStack == null || aMaterial == null || aMaterialAmount <= 0 || aMaterial.contains(TD.Properties.NO_SMELTING) || (aMaterialAmount > U && aMaterial.contains(TD.Atomic.METAL))) return;
//		aMaterialAmount /= aStack.stackSize;
//		if (aAllowAlloySmelter)
//			GT_ModHandler.addSmeltingAndAlloySmeltingRecipe(UT.Stacks.copyAmount(1, aStack), OM.ingot(aMaterial.mSmeltInto, aMaterialAmount));
//		else
//			GT_ModHandler.addSmeltingRecipe(UT.Stacks.copyAmount(1, aStack), OM.ingot(aMaterial.mSmeltInto, aMaterialAmount));
	}
	
	public static void registerReverseArcSmelting(ItemStack aStack, OreDictMaterial aMaterial, long aMaterialAmount, OreDictMaterialStack aByProduct01, OreDictMaterialStack aByProduct02, OreDictMaterialStack aByProduct03) {
//		registerReverseArcSmelting(aStack, new OreDictItemData(aMaterial==null?null:OM.stack(aMaterial, aMaterialAmount), aByProduct01, aByProduct02, aByProduct03));
	}
	
	public static void registerReverseArcSmelting(ItemStack aStack, OreDictItemData aData) {
//		if (aStack == null || aData == null) return;
//		aData = new OreDictItemData(aData);
//		
//		if (!aData.hasValidMaterialData()) return;
//		
//		for (OreDictMaterialStack tMaterial : aData.getAllMaterialStacks()) {
//			if (tMaterial.mMaterial.contains(TD.Properties.UNBURNABLE)) {
//				tMaterial.mMaterial = tMaterial.mMaterial.mSmeltInto.mArcSmeltInto;
//				continue;
//			}
//			if (tMaterial.mMaterial.contains(TD.Properties.EXPLOSIVE)) {
//				tMaterial.mMaterial = MT.Ash;
//				tMaterial.mAmount /= 4;
//				continue;
//			}
//			if (tMaterial.mMaterial.contains(TD.Properties.FLAMMABLE)) {
//				tMaterial.mMaterial = MT.Ash;
//				tMaterial.mAmount /= 2;
//				continue;
//			}
//			if (tMaterial.mMaterial.contains(TD.Properties.NO_SMELTING)) {
//				tMaterial.mAmount = 0;
//				continue;
//			}
//			if (tMaterial.mMaterial.contains(TD.Atomic.METAL)) {
//				tMaterial.mMaterial = tMaterial.mMaterial.mSmeltInto.mArcSmeltInto;
//				continue;
//			}
//			tMaterial.mAmount = 0;
//		}
//		
//		aData = new OreDictItemData(aData);
//		
//		if (aData.mByProducts.length > 3) for (OreDictMaterialStack tMaterial : aData.getAllMaterialStacks()) if (tMaterial.mMaterial == MT.Ash) tMaterial.mAmount = 0;
//		
//		aData = new OreDictItemData(aData);
//		
//		if (!aData.hasValidMaterialData()) return;
//		
//		long tAmount = 0; for (OreDictMaterialStack tMaterial : aData.getAllMaterialStacks()) tAmount += tMaterial.mAmount * tMaterial.mMaterial.getMass();
//		
//		RA.addArcFurnaceRecipe(aStack, new ItemStack[] {OM.ingotOrDust(aData.mMaterial), OM.ingotOrDust(aData.getByProduct(0)), OM.ingotOrDust(aData.getByProduct(1)), OM.ingotOrDust(aData.getByProduct(2))}, null, (int)Math.max(16, tAmount / U), 96);
	}
	
	public static void registerReverseMacerating(ItemStack aStack, OreDictMaterial aMaterial, long aMaterialAmount, OreDictMaterialStack aByProduct01, OreDictMaterialStack aByProduct02, OreDictMaterialStack aByProduct03, boolean aAllowHammer) {
		registerReverseMacerating(aStack, new OreDictItemData(aMaterial==null?null:OM.stack(aMaterial, aMaterialAmount), aByProduct01, aByProduct02, aByProduct03), aAllowHammer);
	}
	
	public static void registerReverseMacerating(ItemStack aStack, OreDictItemData aData, boolean aAllowHammer) {
//		if (aStack == null || aData == null) return;
//		aData = new OreDictItemData(aData);
//		
//		if (!aData.hasValidMaterialData()) return;
//		
//		for (OreDictMaterialStack tMaterial : aData.getAllMaterialStacks()) tMaterial.mMaterial = tMaterial.mMaterial.mMacerateInto;
//		
//		aData = new OreDictItemData(aData);
//		
//		if (!aData.hasValidMaterialData()) return;
//		
//		long tAmount = 0; for (OreDictMaterialStack tMaterial : aData.getAllMaterialStacks()) tAmount += tMaterial.mAmount * tMaterial.mMaterial.getMass();
//		
//		RA.addPulveriserRecipe(aStack, new ItemStack[] {OM.dust(aData.mMaterial), OM.dust(aData.getByProduct(0)), OM.dust(aData.getByProduct(1)), OM.dust(aData.getByProduct(2))}, null, (int)Math.max(16, tAmount / U), 4);
//		
//		if (aAllowHammer) for (OreDictMaterialStack tMaterial : aData.getAllMaterialStacks()) if (tMaterial.mMaterial.contains(TD.Properties.CRYSTAL) && !tMaterial.mMaterial.contains(TD.Atomic.METAL)) {
//			if (RA.addForgeHammerRecipe(UT.Stacks.copyAmount(1, aStack), OM.gem(aData.mMaterial), 200, 32)) break;
//		}
//		ItemStack tDust = OM.dust(aData.mMaterial);
//		if (tDust != null && GT_ModHandler.addPulverisationRecipe(UT.Stacks.copyAmount(1, aStack), tDust, OM.dust(aData.getByProduct(0)), 100, OM.dust(aData.getByProduct(1)), 100, true)) {
//			if (ConfigsGT.sThaumcraftCompat != null) ConfigsGT.sThaumcraftCompat.addCrucibleRecipe(IThaumcraftCompat.ADVANCEDENTROPICPROCESSING, aStack, tDust, Arrays.asList(new TC_AspectStack(TC.PERDITIO, Math.max(1, (aData.mMaterial.mAmount * 2) / U))));
//		}
	}
	
	/**
	 * You give this Function a Material and it will scan almost everything for adding recycling Recipes
	 * 
	 * @param aMat a Material, for example an Ingot or a Gem.
	 * @param aOutput the Dust you usually get from macerating aMat
	 * @param aRecipeReplacing allows to replace the Recipe with a Plate variant
	 */
	public static synchronized void registerUsagesForMaterials(ItemStack aMat, String aPlate, boolean aRecipeReplacing) {
		if (aMat == null) return;
		aMat = UT.Stacks.copy(aMat);
		ItemStack tStack;
		OreDictItemData aItemData = OM.data(aMat);
		if (aItemData == null || aItemData.mPrefix != OP.ingot) aPlate = null;
		if (aPlate != null && OreDictManager.INSTANCE.getFirstOre(aPlate, 1) == null) aPlate = null;
		
		INGT.func_150996_a(aMat.getItem());
		INGT.stackSize = 1;
		Items.feather.setDamage(INGT, Items.feather.getDamage(aMat));
		
		STCK.func_150996_a(new ItemStack(Blocks.dirt).getItem());
		STCK.stackSize = 1;
		Items.feather.setDamage(STCK, 0);
		
		for (RecipeReplacer aRecipe : sRecipes) {
			int tAmount1 = 0;
			for (ItemStack tMat : aRecipe.mRecipe) {
				if (tMat == INGT) tAmount1++;
			}
			if (aItemData != null && aItemData.hasValidPrefixMaterialData()) for (ItemStack tCrafted : GT_ModHandler.getRecipeOutputs(aRecipe.mRecipe)) {
				OM.data(tCrafted, new OreDictItemData(aItemData.mMaterial.mMaterial, aItemData.mMaterial.mAmount * tAmount1));
			}
		}
		
		for (OreDictMaterial tMaterial : OP.stick.mRegisteredMaterials) {
			ItemStack tMt2 = OM.get(OP.stick, tMaterial, 1);
			if (tMt2 != null) {
				STCK.func_150996_a(tMt2.getItem());
				STCK.stackSize = 1;
				Items.feather.setDamage(STCK, Items.feather.getDamage(tMt2));
				
				for (RecipeReplacer aRecipe : sRecipes) {
					int tAmount1 = 0, tAmount2 = 0;
					for (ItemStack tMat : aRecipe.mRecipe) {
						if (tMat == INGT) tAmount1++;
						if (tMat == STCK) tAmount2++;
					}
					for (OreDictMaterial tKey : (aItemData==null ? new OreDictMaterial[] {null} : new OreDictMaterial[] {aItemData.mMaterial.mMaterial, null})) {
						for (ItemStack tCrafted : GT_ModHandler.getVanillyToolRecipeOutputs(tKey, aRecipe.mRecipe)) {
							if (aItemData != null && aItemData.hasValidPrefixMaterialData()) OM.data(tCrafted, new OreDictItemData(aItemData.mMaterial.mMaterial, aItemData.mMaterial.mAmount * tAmount1, OM.stack(tMaterial, OP.stick.mAmount * tAmount2)).setUseVanillaDamage());
							
							if (aRecipeReplacing && aPlate != null && aRecipe.mShape != null && aRecipe.mShape.length > 0) {
								assert aItemData != null;
								if (ConfigsGT.RECIPES.get(ConfigCategories.Recipes.recipereplacements, aItemData.mMaterial.mMaterial+"."+aRecipe.mName, true)) {
									if (null != (tStack = GT_ModHandler.removeRecipe(aRecipe.mRecipe))) {
										switch (aRecipe.mShape.length) {
										case  1: GT_ModHandler.addCraftingRecipe(tStack, GT_ModHandler.RecipeBits.BUFFERED, new Object[] {aRecipe.mShape[0]											, s_P.charAt(0), aPlate, s_R.charAt(0), OP.stick.get(tMaterial), s_I.charAt(0), aItemData}); break;
										case  2: GT_ModHandler.addCraftingRecipe(tStack, GT_ModHandler.RecipeBits.BUFFERED, new Object[] {aRecipe.mShape[0], aRecipe.mShape[1]						, s_P.charAt(0), aPlate, s_R.charAt(0), OP.stick.get(tMaterial), s_I.charAt(0), aItemData}); break;
										default: GT_ModHandler.addCraftingRecipe(tStack, GT_ModHandler.RecipeBits.BUFFERED, new Object[] {aRecipe.mShape[0], aRecipe.mShape[1], aRecipe.mShape[2]	, s_P.charAt(0), aPlate, s_R.charAt(0), OP.stick.get(tMaterial), s_I.charAt(0), aItemData}); break;
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	public static class RecipeReplacer {
		public final ItemStack[] mRecipe;
		public final String mName;
		public final String[] mShape;
		
		public RecipeReplacer(ItemStack[] aRecipe, String aName, String... aShape) {
			mRecipe = aRecipe;
			mName = aName;
			mShape = aShape;
		}
	}
	
	private static final ItemStack INGT = new ItemStack(Blocks.dirt, 1, 0), STCK = new ItemStack(Blocks.dirt, 1, 0);
	private static final String s_H = "h", s_F = "f", s_I = "I", s_P = "P", s_R = "R";
	
	public static final RecipeReplacer[] sRecipes = {
		  new RecipeReplacer(new ItemStack[]
				{INGT, null, INGT
				,INGT, INGT, INGT
				,null, INGT, null}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{INGT, null, INGT
				,INGT, null, INGT
				,INGT, INGT, INGT}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{null, INGT, null
				,INGT, INGT, INGT
				,INGT, null, INGT}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{INGT, INGT, INGT
				,INGT, null, INGT
				,null, null, null}
				, "Helmet"
				, s_P+s_P+s_P
				, s_P+s_H+s_P)
		, new RecipeReplacer(new ItemStack[]
			    {INGT, null, INGT
				,INGT, INGT, INGT
				,INGT, INGT, INGT}
				, "ChestPlate"
				, s_P+s_H+s_P
				, s_P+s_P+s_P
				, s_P+s_P+s_P)
		, new RecipeReplacer(new ItemStack[]
			    {INGT, INGT, INGT
				,INGT, null, INGT
				,INGT, null, INGT}
				, "Pants"
				, s_P+s_P+s_P
				, s_P+s_H+s_P
				, s_P+" "+s_P)
		, new RecipeReplacer(new ItemStack[]
			    {null, null, null
				,INGT, null, INGT
				,INGT, null, INGT}
				, "Boots"
				, s_P+" "+s_P
				, s_P+s_H+s_P)
		, new RecipeReplacer(new ItemStack[]
				{null, INGT, null
				,null, INGT, null
				,null, STCK, null}
				, "Sword"
				, " "+s_P+" "
				, s_F+s_P+s_H
				, " "+s_R+" ")
		, new RecipeReplacer(new ItemStack[]
			    {INGT, INGT, INGT
				,null, STCK, null
				,null, STCK, null}
				, "Pickaxe"
				, s_P+s_I+s_I
				, s_F+s_R+s_H
				, " "+s_R+" ")
		, new RecipeReplacer(new ItemStack[]
			    {null, INGT, null
				,null, STCK, null
				,null, STCK, null}
				, "Shovel"
				, s_F+s_P+s_H
				, " "+s_R+" "
				, " "+s_R+" ")
		, new RecipeReplacer(new ItemStack[]
			    {INGT, INGT, null
				,INGT, STCK, null
				,null, STCK, null}
				, "Axe"
				, s_P+s_I+s_H
				, s_P+s_R+" "
				, s_F+s_R+" ")
		, new RecipeReplacer(new ItemStack[]
			    {null, INGT, INGT
				,null, STCK, INGT
				,null, STCK, null}
				, "Axe"
				, s_P+s_I+s_H
				, s_P+s_R+" "
				, s_F+s_R+" ")
		, new RecipeReplacer(new ItemStack[]
			    {INGT, INGT, null
				,null, STCK, null
				,null, STCK, null}
				, "Hoe"
				, s_P+s_I+s_H
				, s_F+s_R+" "
				, " "+s_R+" ")
		, new RecipeReplacer(new ItemStack[]
			    {null, INGT, INGT
				,null, STCK, null
				,null, STCK, null}
				, "Hoe"
				, s_P+s_I+s_H
				, s_F+s_R+" "
				, " "+s_R+" ")
		, new RecipeReplacer(new ItemStack[]
			    {null, INGT, null
				,INGT, null, null
				,null, INGT, STCK}
				, "Sickle"
				, " "+s_P+" "
				, s_P+s_F+" "
				, s_H+s_P+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {null, INGT, null
				,null, null, INGT
				,STCK, INGT, null}
				, "Sickle"
				, " "+s_P+" "
				, s_P+s_F+" "
				, s_H+s_P+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {null, INGT, null
				,INGT, null, INGT
				,null, null, STCK}
				, "Sickle"
				, " "+s_P+" "
				, s_P+s_F+" "
				, s_H+s_P+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {null, INGT, null
				,INGT, null, INGT
				,STCK, null, null}
				, "Sickle"
				, " "+s_P+" "
				, s_P+s_F+" "
				, s_H+s_P+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {null, STCK, null
				,null, INGT, null
				,null, INGT, null}
				, "Sword"
				, " "+s_R+" "
				, s_F+s_P+s_H
				, " "+s_P+" ")
		, new RecipeReplacer(new ItemStack[]
			    {null, STCK, null
				,null, STCK, null
				,INGT, INGT, INGT}
				, "Pickaxe"
				, " "+s_R+" "
				, s_F+s_R+s_H
				, s_P+s_I+s_I)
		, new RecipeReplacer(new ItemStack[]
			    {null, STCK, null
				,null, STCK, null
				,null, INGT, null}
				, "Shovel"
				, " "+s_R+" "
				, " "+s_R+" "
				, s_F+s_P+s_H)
		, new RecipeReplacer(new ItemStack[]
			    {null, STCK, null
				,INGT, STCK, null
				,INGT, INGT, null}
				, "Axe"
				, s_F+s_R+" "
				, s_P+s_R+" "
				, s_P+s_I+s_H)
		, new RecipeReplacer(new ItemStack[]
			    {null, STCK, null
				,null, STCK, INGT
				,null, INGT, INGT}
				, "Axe"
				, s_F+s_R+" "
				, s_P+s_R+" "
				, s_P+s_I+s_H)
		, new RecipeReplacer(new ItemStack[]
			    {null, STCK, null
				,null, STCK, null
				,INGT, INGT, null}
				, "Hoe"
				, " "+s_R+" "
				, s_F+s_R+" "
				, s_P+s_I+s_H)
		, new RecipeReplacer(new ItemStack[]
			    {INGT, null, null
				,null, STCK, null
				,null, null, STCK}
				, "Hoe"
				, " "+s_R+" "
				, s_F+s_R+" "
				, s_P+s_I+s_H)
		, new RecipeReplacer(new ItemStack[]
			    {null, null, INGT
				,null, STCK, null
				,STCK, null, null}
				, "Spear"
				, s_P+s_H+" "
				, s_F+s_R+" "
				, " "+" "+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {INGT, null, null
				,null, STCK, null
				,null, null, null}
				, "Spear"
				, s_P+s_H+" "
				, s_F+s_R+" "
				, " "+" "+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {null, null, INGT
				,null, STCK, null
				,null, null, null}
				, "Knife"
				, s_H+s_P
				, s_R+s_F)
		, new RecipeReplacer(new ItemStack[]
			    {INGT, STCK, null
				,null, null, null
				,null, null, null}
				, "Knife"
				, s_F+s_H
				, s_P+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {STCK, INGT, null
				,null, null, null
				,null, null, null}
				, "Knife"
				, s_F+s_H
				, s_P+s_R)
		, new RecipeReplacer(new ItemStack[]
			    {INGT, null, null
				,STCK, null, null
				,null, null, null}
				, "Knife"
				, s_P+s_F
				, s_R+s_H)
		, new RecipeReplacer(new ItemStack[]
			    {STCK, null, null
				,INGT, null, null
				,null, null, null}
				, "Knife"
				, s_P+s_F
				, s_R+s_H)
		, new RecipeReplacer(new ItemStack[]
			    {INGT, INGT, INGT
				,INGT, INGT, INGT
				,null, STCK, null}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{INGT, INGT, null
				,INGT, INGT, STCK
				,INGT, INGT, null}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{null, INGT, INGT
				,STCK, INGT, INGT
				,null, INGT, INGT}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{null, STCK, null
				,INGT, INGT, INGT
				,INGT, INGT, INGT}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{INGT, INGT, INGT
				,INGT, STCK, INGT
				,null, STCK, null}
				, "WarAxe"
				, s_P+s_P+s_P
				, s_P+s_R+s_P
				, s_F+s_R+s_H)
		, new RecipeReplacer(new ItemStack[]
				{INGT, INGT, null
				,INGT, STCK, STCK
				,INGT, INGT, null}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{null, INGT, INGT
				,STCK, STCK, INGT
				,null, INGT, INGT}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{null, STCK, null
				,INGT, STCK, INGT
				,INGT, INGT, INGT}
				, null)
		, new RecipeReplacer(new ItemStack[]
				{INGT, null, null
				,null, INGT, null
				,null, null, null}
				, "Shears"
				, s_H+s_P
				, s_P+s_F)
		, new RecipeReplacer(new ItemStack[]
				{null, INGT, null
				,INGT, null, null
				,null, null, null}
				, "Shears"
				, s_H+s_P
				, s_P+s_F)
		, new RecipeReplacer(new ItemStack[]
				{INGT, INGT, null
				,STCK, null, INGT
				,STCK, null, null}
				, "Scythe"
				, s_I+s_P+s_H
				, s_R+s_F+s_P
				, s_R+" "+" ")
		, new RecipeReplacer(new ItemStack[]
				{null, INGT, INGT
				,INGT, null, STCK
				,null, null, STCK}
				, "Scythe"
				, s_H+s_P+s_I
				, s_P+s_F+s_R
				, " "+" "+s_R)
	};
}