package gregapi.tileentity;

import static gregapi.data.CS.*;
import gregapi.GT_API;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.TagData;
import gregapi.data.CS.ModIDs;
import gregapi.data.CS.SFX;
import gregapi.data.TD;
import gregapi.log.GT_Log;
import gregapi.util.UT;
import ic2.api.energy.event.EnergyTileLoadEvent;
import ic2.api.energy.event.EnergyTileUnloadEvent;
import ic2.api.energy.tile.IEnergyTile;

import java.util.Collection;

import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.fluids.*;
import cpw.mods.fml.common.Optional;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 * 
 * The Functions all TileEntities should have.
 */
public abstract class TileEntityBase1 extends TileEntity implements ITileEntity {
	/** If this TileEntity checks for the Chunk to be loaded before returning World based values. If this is set to T, this TileEntity will not cause worfin' Chunks, uhh I mean orphan Chunks. */
	public boolean mIgnoreUnloadedChunks = T;
	
	/** This Variable checks if this TileEntity is dead, because Minecraft is too stupid to have proper TileEntity unloading. */
	public boolean mIsDead = F;
	
	/** This Variable checks if this TileEntity should refresh when the Block is being set. That way you can turn this check off any time you need it. */
	public boolean mShouldRefresh = T;
	
	/** This Variable is for a buffered Block Update. */
	public boolean mDoesBlockUpdate = F;
	
	/** This Variable is for the IC2 E-net. */
	public boolean mIsAddedToEnet = F, mDoEnetCheck = T;
	
	/** If this TileEntity is ticking at all */
	public final boolean mIsTicking;
	
	public TileEntityBase1(boolean aIsTicking) {
		mIsTicking = aIsTicking;
	}
	
    /**
     * YOU MUST HAVE THIS INSIDE YOUR BLOCK CODE!!!
     * 
     * public void onNeighborChange(IBlockAccess aWorld, int aX, int aY, int aZ, int aTileX, int aTileY, int aTileZ) {
     *     TileEntity tTileEntity = aWorld.getTileEntity(aX, aY, aZ);
     *     if (tTileEntity instanceof ITileEntity) ((ITileEntity)tTileEntity).onAdjacentBlockChange(aTileX, aTileY, aTileZ);
     * }
     * 
	 * public void onNeighborBlockChange(World aWorld, int aX, int aY, int aZ, Block aBlock) {
	 * 	   TileEntity tTileEntity = aWorld.getTileEntity(aX, aY, aZ);
	 *     if (tTileEntity instanceof ITileEntity) ((ITileEntity)tTileEntity).onAdjacentBlockChange(aX, aY, aZ);
     * }
     */
	@Override
	public void onAdjacentBlockChange(int aTileX, int aTileY, int aTileZ) {
		//
    }
	
	@Override public World getWorld() {return worldObj;}
	@Override public int getX() {return xCoord;}
	@Override public int getY() {return yCoord;}
	@Override public int getZ() {return zCoord;}
	@Override public int getOffsetX(byte aSide, int aMultiplier) {return xCoord + OFFSETS_X[aSide] * aMultiplier;}
    @Override public int getOffsetY(byte aSide, int aMultiplier) {return yCoord + OFFSETS_Y[aSide] * aMultiplier;}
    @Override public int getOffsetZ(byte aSide, int aMultiplier) {return zCoord + OFFSETS_Z[aSide] * aMultiplier;}
    @Override public boolean isServerSide() {return !worldObj.isRemote;}
	@Override public boolean isClientSide() {return  worldObj.isRemote;}
	@Override public boolean openGUI(EntityPlayer aPlayer) {return openGUI(aPlayer, 0);}
	@Override public boolean openGUI(EntityPlayer aPlayer, int aID) {if (aPlayer == null) return F; aPlayer.openGui(GT_API.instance, aID, worldObj, xCoord, yCoord, zCoord); return T;}
    @Override public int getRandomNumber(int aRange) {return worldObj.rand.nextInt(aRange);}
	@Override public BiomeGenBase getBiome(int aX, int aZ) {return worldObj.getBiomeGenForCoords(aX, aZ);}
	@Override public BiomeGenBase getBiome() {return getBiome(xCoord, zCoord);}
    @Override public Block getBlockOffset(int aX, int aY, int aZ) {return getBlock(xCoord+aX, yCoord+aY, zCoord+aZ);}
    @Override public Block getBlockAtSide(byte aSide) {return getBlockAtSideAndDistance(aSide, 1);}
    @Override public Block getBlockAtSideAndDistance(byte aSide, int aDistance) {return getBlock(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
    @Override public byte getMetaDataOffset(int aX, int aY, int aZ) {return getMetaData(xCoord+aX, yCoord+aY, zCoord+aZ);}
    @Override public byte getMetaDataAtSide(byte aSide) {return getMetaDataAtSideAndDistance(aSide, 1);}
    @Override public byte getMetaDataAtSideAndDistance(byte aSide, int aDistance) {return getMetaData(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
    @Override public byte getLightLevelOffset(int aX, int aY, int aZ) {return getLightLevel(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public byte getLightLevelAtSide(byte aSide) {return getLightLevelAtSideAndDistance(aSide, 1);}
	@Override public byte getLightLevelAtSideAndDistance(byte aSide, int aDistance) {return getLightLevel(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public boolean getOpacityOffset(int aX, int aY, int aZ) {return getOpacity(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public boolean getOpacityAtSide(byte aSide) {return getOpacityAtSideAndDistance(aSide, 1);}
	@Override public boolean getOpacityAtSideAndDistance(byte aSide, int aDistance) {return getOpacity(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public boolean getSkyOffset(int aX, int aY, int aZ) {return getSky(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public boolean getSkyAtSide(byte aSide) {return getSkyAtSideAndDistance(aSide, 1);}
	@Override public boolean getSkyAtSideAndDistance(byte aSide, int aDistance) {return getSky(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public boolean getAirOffset(int aX, int aY, int aZ) {return getAir(xCoord+aX, yCoord+aY, zCoord+aZ);}
	@Override public boolean getAirAtSide(byte aSide) {return getAirAtSideAndDistance(aSide, 1);}
	@Override public boolean getAirAtSideAndDistance(byte aSide, int aDistance) {return getAir(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
	@Override public TileEntity getTileEntityOffset(int aX, int aY, int aZ) {return getTileEntity(xCoord+aX, yCoord+aY, zCoord+aZ);}
    @Override public TileEntity getTileEntityAtSideAndDistance(byte aSide, int aDistance) {return getTileEntity(getOffsetX(aSide, aDistance), getOffsetY(aSide, aDistance), getOffsetZ(aSide, aDistance));}
    @Override public DelegatorTileEntity<TileEntity			> getAdjacentTileEntity		(byte aSide) {TileEntity tTileEntity = getTileEntityAtSideAndDistance(aSide, 1); return tTileEntity instanceof ITileEntityDelegating ? ((ITileEntityDelegating)tTileEntity).getDelegateTileEntity(OPPOSITES[aSide]) : new DelegatorTileEntity<TileEntity>(tTileEntity, tTileEntity!=null?tTileEntity.getWorldObj():worldObj, tTileEntity!=null?tTileEntity.xCoord:getOffsetX(aSide, 1), tTileEntity!=null?tTileEntity.yCoord:getOffsetY(aSide, 1), tTileEntity!=null?tTileEntity.zCoord:getOffsetZ(aSide, 1), OPPOSITES[aSide]);}
    @Override public DelegatorTileEntity<IInventory			> getAdjacentInventory		(byte aSide) {DelegatorTileEntity tDelegator = getAdjacentTileEntity(aSide); return new DelegatorTileEntity<IInventory			>(tDelegator.mTileEntity instanceof IInventory			?(IInventory		)tDelegator.mTileEntity:null, tDelegator);}
    @Override public DelegatorTileEntity<ISidedInventory	> getAdjacentSidedInventory	(byte aSide) {DelegatorTileEntity tDelegator = getAdjacentTileEntity(aSide); return new DelegatorTileEntity<ISidedInventory		>(tDelegator.mTileEntity instanceof ISidedInventory		?(ISidedInventory	)tDelegator.mTileEntity:null, tDelegator);}
    @Override public DelegatorTileEntity<IFluidHandler		> getAdjacentTank			(byte aSide) {DelegatorTileEntity tDelegator = getAdjacentTileEntity(aSide); return new DelegatorTileEntity<IFluidHandler		>(tDelegator.mTileEntity instanceof IFluidHandler		?(IFluidHandler		)tDelegator.mTileEntity:null, tDelegator);}
    @Override public DelegatorTileEntity<ITileEntityEnergy	> getAdjacentEnergyTile		(byte aSide) {DelegatorTileEntity tDelegator = getAdjacentTileEntity(aSide); return new DelegatorTileEntity<ITileEntityEnergy	>(tDelegator.mTileEntity instanceof ITileEntityEnergy	?(ITileEntityEnergy	)tDelegator.mTileEntity:null, tDelegator);}
    
    @Override
    public Block getBlock(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return Blocks.air;
    	return worldObj.getBlock(aX, aY, aZ);
    }
    
    @Override
    public byte getMetaData(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return 0;
    	return (byte)worldObj.getBlockMetadata(aX, aY, aZ);
    }
    
    @Override
    public byte getLightLevel(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return 0;
    	return (byte)(worldObj.getLightBrightness(aX, aY, aZ)*15);
    }
    
    @Override
    public boolean getSky(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return T;
    	return worldObj.canBlockSeeTheSky(aX, aY, aZ);
    }
    
    @Override
    public boolean getOpacity(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return F;
    	return worldObj.getBlock(aX, aY, aZ).isOpaqueCube();
    }
    
    @Override
    public boolean getAir(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return T;
    	return worldObj.getBlock(aX, aY, aZ).isAir(worldObj, aX, aY, aZ);
    }
    
    @Override
    public TileEntity getTileEntity(int aX, int aY, int aZ) {
    	if (mIgnoreUnloadedChunks && crossedChunkBorder(aX, aZ) && !worldObj.blockExists(aX, aY, aZ)) return null;
    	return UT.Worlds.getTileEntity(worldObj, aX, aY, aZ, T);
    }
    
	@Override
	public void sendBlockEvent(byte aID, byte aValue) {
		worldObj.addBlockEvent(xCoord, yCoord, zCoord, getBlockOffset(0, 0, 0), aID, aValue);
	}
	
    @Override
    public boolean isDead() {
    	return mIsDead;
    }
    
    @Override
    public void invalidate() {
    	super.invalidate();
    	setDead();
    }
    
    @Override
    public void validate() {
    	super.validate();
    	setAlive();
    }
    
    @Override
    public void onChunkUnload() {
    	super.onChunkUnload();
    	setDead();
    }
    
	@Optional.Method(modid = ModIDs.IC2)
	private void loadIntoEnet() {
		if (this instanceof IEnergyTile && (isEnergyType(TD.Energy.EU, SIDE_ANY, T) || isEnergyType(TD.Energy.EU, SIDE_ANY, F))) {
			MinecraftForge.EVENT_BUS.post(new EnergyTileLoadEvent((IEnergyTile)this));
			mIsAddedToEnet = T;
		} else {
			mDoEnetCheck = F;
		}
	}
	
	@Optional.Method(modid = ModIDs.IC2)
	private void unloadFromEnet() {
		MinecraftForge.EVENT_BUS.post(new EnergyTileUnloadEvent((IEnergyTile)this));
		mIsAddedToEnet = F;
	}
	
    protected void setDead() {
    	if (!mIsDead) {
        	mIsDead = T;
        	if (!worldObj.isRemote && mIsAddedToEnet) try {unloadFromEnet();} catch(Throwable e) {/**/}
    	}
    }
    
    protected void setAlive() {
        mIsDead = F;
    }
    
    @Override
    public void updateEntity() {
    	// Well, if the TileEntity gets ticked, it is alive.
    	setAlive();
    	
    	if (!worldObj.isRemote && !mIsAddedToEnet && mDoEnetCheck) try {loadIntoEnet();} catch(Throwable e) {/**/}
    	
    	if (mExplosionStrength > 0.0F) {
    		worldObj.setBlockToAir(xCoord, yCoord, zCoord);
    		if (mExplosionStrength < 1) {
    			UT.Sounds.send(worldObj, SFX.MC_EXPLODE, 1.0F, 1.0F, xCoord, yCoord, zCoord);
    		} else {
    			worldObj.newExplosion(null, xCoord, yCoord, zCoord, mExplosionStrength, F, T);
    		}
        	setDead();
    		return;
    	}
    	
    	if (mDoesBlockUpdate) {
    		worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord, zCoord, worldObj.getBlock(xCoord, yCoord, zCoord));
    		mDoesBlockUpdate = F;
    	}
    }
    
	@Override
	public long getTimer() {
		return 0;
	}
	
	@Override
    public boolean canUpdate() {
        return mIsTicking && mShouldRefresh;
    }
	
    @Override
    public boolean shouldRefresh(Block aOldBlock, Block aNewBlock, int aOldMeta, int aNewMeta, World aWorld, int aX, int aY, int aZ) {
        return mShouldRefresh;
    }
	
    /** Simple Function to prevent Block Updates from happening multiple times within the same Tick. */
    protected final void causeBlockUpdate() {
    	if (mIsTicking) mDoesBlockUpdate = T; else worldObj.notifyBlocksOfNeighborChange(xCoord, yCoord, zCoord, worldObj.getBlock(xCoord, yCoord, zCoord));
    }
    
	protected final boolean crossedChunkBorder(int aX, int aZ) {
		return aX >> 4 != xCoord >> 4 || aZ >> 4 != zCoord >> 4;
	}
	
	protected float mExplosionStrength = 0.0F;
	
	protected void explode() {
		// Seems to be a reasonable Default Explosion. This Function can of course be overridden.
		explode(4.0F);
	}
	
	protected void explode(double aStrength) {
		mExplosionStrength = (float)Math.max(aStrength, mExplosionStrength);
		if (!mIsTicking) {
    		worldObj.setBlockToAir(xCoord, yCoord, zCoord);
    		if (mExplosionStrength < 1) {
    			UT.Sounds.send(worldObj, SFX.MC_EXPLODE, 1.0F, 1.0F, xCoord, yCoord, zCoord);
    		} else {
    			worldObj.newExplosion(null, xCoord, yCoord, zCoord, mExplosionStrength, F, T);
    		}
		}
	}
	
	protected void overcharge(long aVoltage, TagData aEnergyType) {
		// Only explode if allowed
		if (OVERCHARGE_EXPLOSIONS) {
			if (TD.Energy.ALL_EXPLODING.contains(aEnergyType)) {
				explode(UT.Code.tier(Math.abs(aVoltage)));
			} else {
				explode(0.1);
			}
		}
		// Yes, I will annoy people with that a lot, even when they disable Explosions.
		UT.Sounds.send(worldObj, SFX.IC_MACHINE_OVERLOAD, 1.0F, 1.0F, xCoord, yCoord, zCoord);
		// The Noise should make the position obvious.
		GT_Log.deb.println("Machine overcharged with: " + aVoltage + " " + aEnergyType.getLocalisedNameLong());
	}
	
	@Override public Object getGUIClient(int aGUIID, EntityPlayer aPlayer) {return null;}
	@Override public Object getGUIServer(int aGUIID, EntityPlayer aPlayer) {return null;}
	@SideOnly(Side.CLIENT) public boolean renderItem(Block aBlock, RenderBlocks aRenderer) {return F;}
	@SideOnly(Side.CLIENT) public boolean renderBlock(Block aBlock, RenderBlocks aRenderer) {return F;}
	
	// A Default implementation of the Fluid Tank behaviour, so that every TileEntity can use this to simplify its Code.
	protected IFluidTank getFluidTankFillable(byte aSide, FluidStack aFluidToFill) {return null;}
	protected IFluidTank getFluidTankDrainable(byte aSide, FluidStack aFluidToDrain) {return null;}
	protected IFluidTank[] getFluidTanks(byte aSide) {return ZL_IFLUIDTANK;}
	
	public int fill(ForgeDirection aDirection, FluidStack aFluid, boolean aDoFill) {
		IFluidTank tTank = getFluidTankFillable(UT.Code.side(aDirection), aFluid);
		return tTank != null ? tTank.fill(aFluid, aDoFill) : 0;
	}
	
	public FluidStack drain(ForgeDirection aDirection, FluidStack aFluid, boolean aDoDrain) {
		IFluidTank tTank = getFluidTankDrainable(UT.Code.side(aDirection), aFluid);
		return tTank != null && aFluid != null && aFluid.amount > 0 && tTank.getFluid() != null && tTank.getFluid().isFluidEqual(aFluid) ? tTank.drain(aFluid.amount, aDoDrain) : null;
	}
	
	public FluidStack drain(ForgeDirection aDirection, int maxDrain, boolean aDoDrain) {
		IFluidTank tTank = getFluidTankDrainable(UT.Code.side(aDirection), null);
		return tTank != null && maxDrain > 0 && tTank.getFluid() != null ? tTank.drain(maxDrain, aDoDrain) : null;
	}
	
	public boolean canFill(ForgeDirection aDirection, Fluid aFluid) {
		IFluidTank tTank = getFluidTankFillable(UT.Code.side(aDirection), new FluidStack(aFluid, 0));
		return tTank != null && (tTank.getFluid() == null || tTank.getFluid().getFluid() == aFluid);
	}
	
	public boolean canDrain(ForgeDirection aDirection, Fluid aFluid) {
		IFluidTank tTank = getFluidTankDrainable(UT.Code.side(aDirection), new FluidStack(aFluid, 0));
		return tTank != null && (tTank.getFluid() != null && tTank.getFluid().getFluid() == aFluid);
	}
	
	public FluidTankInfo[] getTankInfo(ForgeDirection aDirection) {
		IFluidTank[] tTanks = getFluidTanks(UT.Code.side(aDirection));
		if (tTanks == null || tTanks.length <= 0) return ZL_FLUIDTANKINFO;
		FluidTankInfo[] rInfo = new FluidTankInfo[tTanks.length];
		for (int i = 0; i < tTanks.length; i++) rInfo[i] = new FluidTankInfo(tTanks[i]);
		return rInfo;
	}
	
	// A Default implementation of the Energy behaviour.
	
	public long doInject(TagData aEnergyType, byte aSide, long aSize, long aAmount, boolean aDoInject) {return 0;}
	public long doExtract(TagData aEnergyType, byte aSide, long aSize, long aAmount, boolean aDoExtract) {return 0;}
	public boolean isEnergyType(TagData aEnergyType, byte aSide, boolean aEmitting) {return F;}
	public boolean isEnergyCapacitorType(TagData aEnergyType, byte aSide) {return F;}
	public long getEnergyStored(TagData aEnergyType, byte aSide) {return 0;}
	public long getEnergyCapacity(TagData aEnergyType, byte aSide) {return 0;}
	public Collection<TagData> getEnergyTypes(byte aSide) {return new ArrayListNoNulls<TagData>();}
	public Collection<TagData> getEnergyCapacitorTypes(byte aSide) {return new ArrayListNoNulls<TagData>();}
	
	
	public boolean isEnergyEmittingTo(TagData aEnergyType, byte aSide, boolean aTheoretical) {return isEnergyType(aEnergyType, aSide, T) && getSurfaceSizeAttachable(aSide) > 0.0F;}
	public boolean isEnergyAcceptingFrom(TagData aEnergyType, byte aSide, boolean aTheoretical) {return isEnergyType(aEnergyType, aSide, F) && getSurfaceSizeAttachable(aSide) > 0.0F;}
	public synchronized long doEnergyExtraction(TagData aEnergyType, byte aSide, long aSize, long aAmount, boolean aDoExtract) {return isEnergyEmittingTo   (aEnergyType, aSide, F) && aSize != 0 && (TD.Energy.ALL_SIZE_IRRELEVANT.contains(aEnergyType) || Math.abs(aSize) >= getEnergySizeOutputMin(aEnergyType, aSide)) ? doExtract(aEnergyType, aSide, aSize, aAmount, aDoExtract) : 0;}
	public synchronized long doEnergyInjection (TagData aEnergyType, byte aSide, long aSize, long aAmount, boolean aDoInject ) {return isEnergyAcceptingFrom(aEnergyType, aSide, F) && aSize != 0 && (TD.Energy.ALL_SIZE_IRRELEVANT.contains(aEnergyType) || Math.abs(aSize) >= getEnergySizeInputMin (aEnergyType, aSide)) ? doInject (aEnergyType, aSide, aSize, aAmount, aDoInject ) : 0;}
	public long getEnergyOffered(TagData aEnergyType, byte aSide, long aSize) {return 0;}
	public long getEnergySizeOutputRecommended(TagData aEnergyType, byte aSide) {return 0;}
	public long getEnergySizeOutputMin(TagData aEnergyType, byte aSide) {return getEnergySizeOutputRecommended(aEnergyType, aSide) / 2;}
	public long getEnergySizeOutputMax(TagData aEnergyType, byte aSide) {return getEnergySizeOutputRecommended(aEnergyType, aSide) * 2;}
	public long getEnergyDemanded(TagData aEnergyType, byte aSide, long aSize) {return 0;}
	public long getEnergySizeInputRecommended(TagData aEnergyType, byte aSide) {return 0;}
	public long getEnergySizeInputMin(TagData aEnergyType, byte aSide) {return getEnergySizeInputRecommended(aEnergyType, aSide) / 2;}
	public long getEnergySizeInputMax(TagData aEnergyType, byte aSide) {return getEnergySizeInputRecommended(aEnergyType, aSide) * 2;}
	
	// A Default implementation for RF Stuff, so I don't have to implement those Interfaces manually every time I make an RF Emitter or Acceptor.
	
	public boolean canConnectEnergy(ForgeDirection aDirection) {for (TagData tTag : TD.Energy.ALL_RF) if (isEnergyEmittingTo(tTag, UT.Code.side(aDirection), T)) return T; return isEnergyAcceptingFrom(TD.Energy.RF, UT.Code.side(aDirection), T);}
	public int receiveEnergy(ForgeDirection aDirection, int aSize, boolean aSimulate) {return (int)doEnergyInjection(TD.Energy.RF, UT.Code.side(aDirection), aSize, 1, !aSimulate)*aSize;}
	public int extractEnergy(ForgeDirection aDirection, int aSize, boolean aSimulate) {return (int)doEnergyExtraction(TD.Energy.RF, UT.Code.side(aDirection), aSize, 1, !aSimulate)*aSize;}
	public int getEnergyStored(ForgeDirection aDirection) {return UT.Code.bindInt(getEnergyCapacity(TD.Energy.RF, UT.Code.side(aDirection)));}
	public int getMaxEnergyStored(ForgeDirection aDirection) {return UT.Code.bindInt(getEnergyCapacity(TD.Energy.RF, UT.Code.side(aDirection)));}
	
	// A Default implementation for EU Stuff, so I don't have to implement those Interfaces manually every time I make an EU Emitter or Acceptor.
	
	public int getSinkTier() {return UT.Code.tier(getEnergySizeInputRecommended(TD.Energy.EU, SIDE_ANY));}
	public boolean acceptsEnergyFrom(TileEntity aEmitter, ForgeDirection aDirection) {return isEnergyAcceptingFrom(TD.Energy.EU, UT.Code.side(aDirection), T);}
	public double getDemandedEnergy() {return getEnergySizeInputMax(TD.Energy.EU, SIDE_ANY);}
	
	public double injectEnergy(ForgeDirection aDirection, double aAmount, double aSize) {
		long tSize = (long)aAmount / (1+(long)aAmount / (long)aSize);
		return aAmount - (tSize <= 0 ? 0 : tSize * doEnergyInjection(TD.Energy.EU, UT.Code.side(aDirection), tSize, (long)aAmount / tSize, T));
	}
	
	// A Default implementation of the Surface behaviour.
	
	public float getSurfaceSize				(byte aSide) {return 1.0F;}
	public float getSurfaceSizeAttachable	(byte aSide) {return 1.0F;}
	public float getSurfaceDistance			(byte aSide) {return 0.0F;}
	public boolean isSurfaceSolid			(byte aSide) {return T;}
	public boolean isSurfaceOpaque			(byte aSide) {return T;}
	
	public int getFireSpreadSpeed(byte aSide) {return 0;}
	public int getFlammability(byte aSide) {return 0;}
}