package gregapi.tileentity.machines;

import static gregapi.data.CS.*;
import gregapi.GT_API;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_AddToolTips;
import gregapi.block.multitileentity.MultiTileEntityRegistry;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.TagData;
import gregapi.data.LH;
import gregapi.data.LH.Chat;
import gregapi.data.TD;
import gregapi.gui.ContainerClient;
import gregapi.gui.ContainerCommon;
import gregapi.gui.Slot_Output;
import gregapi.network.INetworkHandler;
import gregapi.old.Textures;
import gregapi.recipes.Recipe;
import gregapi.recipes.Recipe.RecipeMap;
import gregapi.render.BlockTextureDefault;
import gregapi.render.BlockTextureMulti;
import gregapi.render.IIconContainer;
import gregapi.render.ITexture;
import gregapi.tileentity.DelegatorTileEntity;
import gregapi.tileentity.ITileEntityEnergy;
import gregapi.tileentity.TileEntityBase6Facing;
import gregapi.util.UT;

import java.util.Collection;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ICrafting;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.ISidedInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 * 
 * This is the Base Class for almost all my Basic Machines. It is almost too simple to use.
 * 
 * In order to create a Basic Machine (Steel Shredder in this example), I use the following NBT Parameters in my MultiTileEntity System:
 * 
 * NBT_MATERIAL						= MT.Steel.mNameInternal
 * NBT_HARDNESS						= 6.0F
 * NBT_RESISTANCE					= 6.0F
 * NBT_COLOR						= UT.Code.getRGBInt(MT.Steel.mRGBaSolid)
 * NBT_INPUT						= 128
 * NBT_GUI							= RES_PATH_GUI+"machines/Shredder"
 * NBT_TEXTURE						= "shredder"
 * NBT_ENERGY_ACCEPTED				= TD.Energy.RU
 * NBT_RECIPEMAP					= RecipeMap.sShredderRecipes.mNameInternal
 * NBT_EFFICIENCY					= 10000
 * NBT_INV_SIDE_INPUT				= SIDE_BITS[SIDE_TOP]
 * NBT_INV_SIDE_AUTO_INPUT			= SIDE_TOP
 * NBT_INV_SIDE_OUTPUT				= SIDE_BITS[SIDE_BOTTOM]
 * NBT_INV_SIDE_AUTO_OUTPUT			= SIDE_BOTTOM
 * NBT_ENERGY_ACCEPTED_SIDES		= SIDE_BITS[SIDE_LEFT]|SIDE_BITS[SIDE_RIGHT]
 */
public class MultiTileEntityBasicMachine extends TileEntityBase6Facing implements ITileEntityEnergy, ISidedInventory, IMTE_AddToolTips {
	protected boolean mActive = F, oActive = F, mRunning = F, oRunning = F, mStateNew = F, mStateOld = F;
	protected boolean[] mEnergyInputAllowed, mItemInputAllowed, mItemOutputAllowed;
	protected byte mEnergyInputs = SIDE_BITS[SIDE_BACK], mItemInputs = SIDE_BITS[SIDE_TOP], mItemOutputs = SIDE_BITS[SIDE_BOTTOM], mOutputBlocked = 0, mAutoInput = SIDE_TOP, mAutoOutput = SIDE_BOTTOM;
	protected short mEfficiency = 10000;
	protected long mEnergy = 0, mInputMin = 16, mInput = 32, mInputMax = 64, mMinEnergy = 0;
	protected TagData mEnergyTypeAccepted = TD.Energy.QU;
	protected Recipe mLastRecipe = null;
	protected ItemStack[] mOutputItems = ZL_ITEMSTACK;
	protected IIconContainer[] mTexturesMaterial, mTexturesInactive, mTexturesActive, mTexturesRunning;
	
	public String mGUITexture = "";
	public RecipeMap mRecipes = RecipeMap.sFurnaceRecipes;
	public long mProgress = 0, mMaxProgress = 0;
	public boolean mSuccessful = F;
	
    @Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
    	mEnergy = aNBT.getLong(NBT_ENERGY);
        if (aNBT.hasKey(NBT_ACTIVE)) mActive = aNBT.getBoolean(NBT_ACTIVE);
        if (aNBT.hasKey(NBT_RUNNING)) mRunning = aNBT.getBoolean(NBT_RUNNING);
        if (aNBT.hasKey(NBT_STATE+".new")) mStateNew = aNBT.getBoolean(NBT_STATE+".new");
        if (aNBT.hasKey(NBT_STATE+".old")) mStateOld = aNBT.getBoolean(NBT_STATE+".old");
        if (aNBT.hasKey(NBT_EFFICIENCY)) mEfficiency = (short)UT.Code.bind(0, 10000, aNBT.getShort(NBT_EFFICIENCY));
        if (aNBT.hasKey(NBT_INPUT)) {mInput = aNBT.getLong(NBT_INPUT); mInputMin = mInput / 2; mInputMax = mInput * 2;}
        if (aNBT.hasKey(NBT_INPUT_MIN)) {mInputMin = aNBT.getLong(NBT_INPUT_MIN);}
        if (aNBT.hasKey(NBT_INPUT_MAX)) {mInputMax = aNBT.getLong(NBT_INPUT_MAX);}
        if (aNBT.hasKey(NBT_MINENERGY)) {mMinEnergy = aNBT.getLong(NBT_MINENERGY);}
        if (aNBT.hasKey(NBT_PROGRESS)) {mProgress = aNBT.getLong(NBT_PROGRESS);}
        if (aNBT.hasKey(NBT_MAXPROGRESS)) {mMaxProgress = aNBT.getLong(NBT_MAXPROGRESS);}
        if (aNBT.hasKey(NBT_INV_SIDE_INPUT)) mItemInputs = aNBT.getByte(NBT_INV_SIDE_INPUT);
        if (aNBT.hasKey(NBT_INV_SIDE_OUTPUT)) mItemOutputs = aNBT.getByte(NBT_INV_SIDE_OUTPUT);
        if (aNBT.hasKey(NBT_INV_SIDE_AUTO_INPUT)) mAutoInput = aNBT.getByte(NBT_INV_SIDE_AUTO_INPUT);
        if (aNBT.hasKey(NBT_INV_SIDE_AUTO_OUTPUT)) mAutoOutput = aNBT.getByte(NBT_INV_SIDE_AUTO_OUTPUT);
        if (aNBT.hasKey(NBT_ENERGY_ACCEPTED)) mEnergyTypeAccepted = TagData.createTagData(aNBT.getString(NBT_ENERGY_ACCEPTED));
        if (aNBT.hasKey(NBT_ENERGY_ACCEPTED_SIDES)) mEnergyInputs = aNBT.getByte(NBT_ENERGY_ACCEPTED_SIDES);
        assignSideArrays();
        
        mOutputItems = new ItemStack[mRecipes.mUsualOutputCount];
		for (int i = 0; i < mOutputItems.length; i++) mOutputItems[i] = UT.Stacks.load(aNBT, NBT_INV_OUTPUT+"."+i);
		
		if (GT_API.api_proxy.isClientSide()) {
	        if (aNBT.hasKey(NBT_GUI)) {
	        	mGUITexture = aNBT.getString(NBT_GUI);
		        if (!mGUITexture.endsWith(".png")) mGUITexture += ".png";
	        }
			if (GT_API.sBlockIcons == null && aNBT.hasKey(NBT_TEXTURE)) {
				String tTextureName = aNBT.getString(NBT_TEXTURE);
				mTexturesMaterial = new IIconContainer[] {
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/colored/bottom"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/colored/top"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/colored/left"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/colored/front"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/colored/right"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/colored/back")};
				mTexturesInactive = new IIconContainer[] {
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay/bottom"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay/top"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay/left"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay/front"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay/right"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay/back")};
				mTexturesActive = new IIconContainer[] {
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_active/bottom"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_active/top"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_active/left"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_active/front"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_active/right"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_active/back")};
				mTexturesRunning = new IIconContainer[] {
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_running/bottom"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_running/top"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_running/left"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_running/front"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_running/right"),
				new Textures.BlockIcons.CustomIcon("machines/basicmachines/"+tTextureName+"/overlay_running/back")};
			} else {
		        if (getMultiTileEntityRegistryID() != W && getMultiTileEntityID() != W) {
					MultiTileEntityBasicMachine tCanonicalTileEntity = (MultiTileEntityBasicMachine)MultiTileEntityRegistry.getRegistry(getMultiTileEntityRegistryID()).getClassContainer(getMultiTileEntityID()).mCanonicalTileEntity;
					mTexturesMaterial = tCanonicalTileEntity.mTexturesMaterial;
					mTexturesInactive = tCanonicalTileEntity.mTexturesInactive;
					mTexturesRunning  = tCanonicalTileEntity.mTexturesRunning;
					mTexturesActive   = tCanonicalTileEntity.mTexturesActive;
		        } else {
		        	mTexturesMaterial = mTexturesInactive = mTexturesRunning = mTexturesActive = new IIconContainer[6];
		        }
	        }
		}
	}
    
    @Override
	public void writeToNBT2(NBTTagCompound aNBT) {
    	super.writeToNBT2(aNBT);
    	UT.NBT.setNumber(aNBT, NBT_ENERGY, mEnergy);
    	UT.NBT.setNumber(aNBT, NBT_MINENERGY, mMinEnergy);
    	UT.NBT.setNumber(aNBT, NBT_PROGRESS, mProgress);
    	UT.NBT.setNumber(aNBT, NBT_MAXPROGRESS, mMaxProgress);
    	UT.NBT.setBoolean(aNBT, NBT_ACTIVE, mActive);
    	UT.NBT.setBoolean(aNBT, NBT_RUNNING, mRunning);
    	UT.NBT.setBoolean(aNBT, NBT_STATE+".new", mStateNew);
    	UT.NBT.setBoolean(aNBT, NBT_STATE+".old", mStateOld);
    	
    	for (int i = 0; i < mOutputItems.length; i++) UT.Stacks.save(aNBT, NBT_INV_OUTPUT+"."+i, mOutputItems[i]);
    }
    
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		aList.add(Chat.CYAN + LH.get(LH.RECIPES) + ": " + Chat.WHITE + LH.get(mRecipes.mNameInternal));
		aList.add(LH.getToolTipEfficiency(mEfficiency));
		LH.addEnergyToolTips(this, aList, mEnergyTypeAccepted, null, null, null);
		
		// TODO: Input/Output Sides (all 5 types)
	}
	
	@Override
	public long onToolClick2(String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, List<String> aChatReturn, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSide, float aHitX, float aHitY, float aHitZ) {
		long rReturn = super.onToolClick2(aTool, aRemainingDurability, aQuality, aPlayer, aChatReturn, aPlayerInventory, aSneaking, aStack, aSide, aHitX, aHitY, aHitZ);
		if (rReturn > 0) return rReturn;
		
		if (worldObj.isRemote) return 0;
		
		if (aTool.equals(TOOL_softhammer)) {
			mProgress = mMinEnergy = mMaxProgress = 0;
			mOutputItems = ZL_ITEMSTACK;
			return 10000;
		}
		return 0;
	}
	
    @Override
    public void onTick(long aTimer, boolean aIsServerSide) {
    	super.onTick(aTimer, aIsServerSide);
    	
    	if (aIsServerSide) {
    		doWork(aTimer);
    	}
    }
    
    @Override
    public boolean onTickCheck(long aTimer) {
    	return mActive != oActive || mRunning != oRunning || super.onTickCheck(aTimer);
    }
    
	@Override
    public void onTickResetChecks(long aTimer, boolean aIsServerSide) {
    	super.onTickResetChecks(aTimer, aIsServerSide);
		oRunning = mRunning;
		oActive = mActive;
    }
    
	@Override
	public boolean receiveDataByte(byte aData, INetworkHandler aNetworkHandler) {
		super.receiveDataByte(aData, aNetworkHandler);
		mRunning = ((aData & 2) != 0);
		mActive = ((aData & 1) != 0);
		return T;
	}
	
	@Override
	public boolean onBlockActivated(EntityPlayer aPlayer, byte aSide, float aHitX, float aHitY, float aHitZ) {
		if (super.onBlockActivated(aPlayer, aSide, aHitX, aHitY, aHitZ)) return T;
		if (isServerSide()) openGUI(aPlayer, aSide);
		return T;
	}
	
	@Override
	public long doInject(TagData aEnergyType, byte aSide, long aSize, long aAmount, boolean aDoInject) {
		mStateNew = (aSize > 0);
		aSize = Math.abs(aSize);
		if (aSize > getEnergySizeInputMax(aEnergyType, aSide)) {
			if (aDoInject) overcharge(aSize, aEnergyType);
			return aAmount;
		}
		long tInput = Math.min(mInputMax - mEnergy, aSize * aAmount), tConsumed = Math.min(aAmount, (tInput/aSize) + (tInput%aSize!=0?1:0));
		if (aDoInject) mEnergy += tConsumed * aSize;
		return tConsumed;
	}
	
	@Override public boolean isEnergyType					(TagData aEnergyType, byte aSide, boolean aEmitting) {return !aEmitting && aEnergyType == mEnergyTypeAccepted;}
	@Override public boolean isEnergyAcceptingFrom			(TagData aEnergyType, byte aSide, boolean aTheoretical) {return mEnergyInputAllowed[aSide] && super.isEnergyAcceptingFrom(aEnergyType, aSide, aTheoretical);}
	@Override public long getEnergySizeInputMin				(TagData aEnergyType, byte aSide) {return mInputMin;}
	@Override public long getEnergySizeInputRecommended		(TagData aEnergyType, byte aSide) {return mInput;}
	@Override public long getEnergySizeInputMax				(TagData aEnergyType, byte aSide) {return mInputMax;}
	@Override public Collection<TagData> getEnergyTypes(byte aSide) {return new ArrayListNoNulls<TagData>(F, mEnergyTypeAccepted);}
	
	// Inventory Stuff
	
	@Override
	public ItemStack[] getDefaultInventory(NBTTagCompound aNBT) {
        if (aNBT.hasKey(NBT_RECIPEMAP)) mRecipes = RecipeMap.RECIPE_MAPS.get(aNBT.getString(NBT_RECIPEMAP));
        ACCESSIBLE_SLOTS = new int[mRecipes.mUsualInputCount + mRecipes.mUsualOutputCount];
        for (int i = 0; i < ACCESSIBLE_SLOTS.length; i++) ACCESSIBLE_SLOTS[i] = i;
		return new ItemStack[mRecipes.mUsualInputCount + mRecipes.mUsualOutputCount];
	}
	
	protected void assignSideArrays() {
        mItemInputAllowed   = new boolean[] {FACE_CONNECTED[FACING_ROTATIONS[mFacing][0]][mItemInputs  ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][1]][mItemInputs  ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][2]][mItemInputs  ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][3]][mItemInputs  ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][4]][mItemInputs  ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][5]][mItemInputs  ], T};
        mItemOutputAllowed  = new boolean[] {FACE_CONNECTED[FACING_ROTATIONS[mFacing][0]][mItemOutputs ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][1]][mItemOutputs ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][2]][mItemOutputs ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][3]][mItemOutputs ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][4]][mItemOutputs ], FACE_CONNECTED[FACING_ROTATIONS[mFacing][5]][mItemOutputs ], T};
        mEnergyInputAllowed = new boolean[] {FACE_CONNECTED[FACING_ROTATIONS[mFacing][0]][mEnergyInputs], FACE_CONNECTED[FACING_ROTATIONS[mFacing][1]][mEnergyInputs], FACE_CONNECTED[FACING_ROTATIONS[mFacing][2]][mEnergyInputs], FACE_CONNECTED[FACING_ROTATIONS[mFacing][3]][mEnergyInputs], FACE_CONNECTED[FACING_ROTATIONS[mFacing][4]][mEnergyInputs], FACE_CONNECTED[FACING_ROTATIONS[mFacing][5]][mEnergyInputs], T};
	}
	
	protected int[] ACCESSIBLE_SLOTS;
	@Override public int[] getAccessibleSlotsFromSide(int aSide) {return ACCESSIBLE_SLOTS;}
	@Override public boolean canDrop(int aInventorySlot) {return T;}
	
	@Override
	public boolean canInsertItem(int aSlot, ItemStack aStack, int aSide) {
		if (!mItemInputAllowed[aSide] || aSlot >= mRecipes.mUsualInputCount) return F;
		for (int i = 0; i < mRecipes.mUsualInputCount; i++) if (UT.Stacks.equal(aStack, slot(i), T)) return i == aSlot;
		return mRecipes.containsInput(aStack);
	}
	
	@Override
	public boolean canExtractItem(int aSlot, ItemStack aStack, int aSide) {
		return mItemOutputAllowed[aSide] && aSlot >= mRecipes.mUsualInputCount && aSlot < mRecipes.mUsualInputCount + mRecipes.mUsualOutputCount;
	}
	
	// Stuff to Override
	
	protected boolean canOutput(Recipe aRecipe) {
    	for (int i = 0, j = mRecipes.mUsualInputCount; i < mRecipes.mUsualOutputCount; i++, j++) {
    		if (aRecipe.mNeedsEmptyOutput) {
        		if (slot(j) != null) {
            		mOutputBlocked++;
            		return F;
        		}
    		} else {
        		if (slot(j) != null && aRecipe.mOutputs[i] != null && (!UT.Stacks.equal(slot(j), aRecipe.mOutputs[i], F) || slot(j).stackSize + aRecipe.mOutputs[i].stackSize > slot(j).getMaxStackSize())) {
            		mOutputBlocked++;
            		return F;
        		}
    		}
    	}
    	// Fluid check
    	return T;
	}
	
	/** return codes for checkRecipe() */
	protected static final int
	DID_NOT_FIND_RECIPE = 0,
	FOUND_RECIPE_BUT_DID_NOT_MEET_REQUIREMENTS = 1,
	FOUND_AND_SUCCESSFULLY_USED_RECIPE = 2;
	
	/**
	 * Override this to check the Recipes yourself, super calls to this could be useful if you just want to add a special case
	 * I thought about Enum too, but Enum doesn't add support for people adding other return Systems.
	 * Funny how Eclipse marks the word Enum as not correctly spelled.
	 * @return see constants above
	 */
	protected int checkRecipe() {
    	if (mRecipes == null) return DID_NOT_FIND_RECIPE;
    	
    	DelegatorTileEntity<IInventory> tTileEntity = null;
    	
    	byte tAutoInput = FACING_ROTATIONS[mFacing][mAutoInput];
		
    	if (SIDES_VALID[tAutoInput]) {
    		tTileEntity = getAdjacentInventory(tAutoInput);
    		if (tTileEntity.mTileEntity == null) tTileEntity = null;
    	}
    	
		ItemStack[] tInputs = new ItemStack[mRecipes.mUsualInputCount];
		for (int i = 0; i < mRecipes.mUsualInputCount; i++) {
			if (tTileEntity != null) UT.Inventories.moveOneItemStackIntoSlot(tTileEntity.mTileEntity, this, tTileEntity.mSideOfTileEntity, i, null, F, (byte)64, (byte)1, (byte)64, (byte)1);
			tInputs[i] = slot(i);
		}
    	
    	Recipe tRecipe = mRecipes.findRecipe(this, mLastRecipe, F, mInputMax, NI, ZL_FLUIDSTACK, tInputs);
    	if (tRecipe == null) return DID_NOT_FIND_RECIPE;
    	if (tRecipe.mCanBeBuffered) mLastRecipe = tRecipe;
        if (!canOutput(tRecipe)) return FOUND_RECIPE_BUT_DID_NOT_MEET_REQUIREMENTS;
        if (!tRecipe.isRecipeInputEqual(true, ZL_FLUIDSTACK, tInputs)) return FOUND_RECIPE_BUT_DID_NOT_MEET_REQUIREMENTS;
        
		for (int i = 0; i < mRecipes.mUsualInputCount; i++) if (slot(i) != null && slot(i).stackSize <= 0) slot(i, null);
        mOutputItems = tRecipe.getOutputs(worldObj.rand);
        mMinEnergy = tRecipe.mEUt;
        long tMultiplier = 1;
        while (mMinEnergy < mInputMin && mMinEnergy * 4 <= mInputMax) {mMinEnergy *= 4; tMultiplier *= 2;}
        mMaxProgress = Math.max(1, UT.Code.units(tMultiplier * tRecipe.mEUt * tRecipe.mDuration, mEfficiency, mEnergyTypeAccepted==TD.Energy.RF?RF_PER_EU*10000:mEnergyTypeAccepted==TD.Energy.MJ?(RF_PER_EU*10000)/RF_PER_MJ:10000, T));
        
//      mOutputFluid = tRecipe.getFluidOutput(0);
//		if (mFluid != null && mFluid.amount <= 0) mFluid = null;
//		if (mInventory[3] != null && mInventory[3].stackSize <= 0) mInventory[3] = null; // Special Slot Stuff.
        return FOUND_AND_SUCCESSFULLY_USED_RECIPE;
    }
	
	protected void doWork(long aTimer) {
		mSuccessful = F;
		if (mEnergy >= mInputMin && mEnergy >= mMinEnergy) {
			mActive = doActive(aTimer, Math.min(mInputMax, mEnergy), mRunning);
			mRunning = T;
		} else {
			mActive = doInactive(aTimer, mRunning);
			mRunning = F;
		}
		mEnergy -= mInputMax; if (mEnergy < 0) mEnergy = 0;
	}
	
	protected boolean doActive(long aTimer, long aEnergy, boolean aReceivedEnoughEnergyBefore) {
		boolean rActive = F;
		
		if (mMaxProgress > 0) {
			rActive = T;
			if (mProgress <= mMaxProgress) mProgress += aEnergy;
    		if (mProgress >= mMaxProgress && (mStateOld&&!mStateNew || !TD.Energy.ALL_ALTERNATING.contains(mEnergyTypeAccepted))) {
    			for (int i = 0; i < mOutputItems.length; i++) for (int j = 0; j < mOutputItems.length; j++) if (addStackToSlot(mRecipes.mUsualInputCount+((j+i)%mOutputItems.length), mOutputItems[i])) break;
//    		    if (mOutputFluid != null) if (getDrainableStack() == null) setDrainableStack(mOutputFluid.copy()); else if (mOutputFluid.isFluidEqual(getDrainableStack())) getDrainableStack().amount += mOutputFluid.amount;
    			mOutputItems = ZL_ITEMSTACK;
//    			mOutputFluid = null;
    			mMinEnergy = 0;
    			mProgress -= mMaxProgress; // this way the energy can temporarily be used on the next processed thing.
    			mMaxProgress = 0;
    			mSuccessful = T;
    			onProcessFinished();
    		}
    	}
		
		mStateOld = mStateNew;
		
		boolean tRemovedOutputFluid = F;
		/*
		if (doesAutoOutputFluids() && getDrainableStack() != null && getFrontFacing() != mMainFacing && (tSucceeded || aTick%20 == 0)) {
	    	IFluidHandler tTank = getITankContainerAtSide(getFrontFacing());
	    	if (tTank != null) {
	    		FluidStack tDrained = drain(1000, false);
	    		if (tDrained != null) {
			    	int tFilledAmount = tTank.fill(ForgeDirection.getOrientation(getBackFacing()), tDrained, false);
			    	if (tFilledAmount > 0) tTank.fill(ForgeDirection.getOrientation(getBackFacing()), drain(tFilledAmount, true), true);
	    		}
	    	}
			if (getDrainableStack() == null) tRemovedOutputFluid = true;
		}
		*/
		
		boolean
		tOutputEmpty = T;
		
		for (int i = mRecipes.mUsualInputCount, j = mRecipes.mUsualInputCount + mRecipes.mUsualOutputCount; i < j; i++) if (slot(i) != null) {tOutputEmpty = F; break;}
		
		if (!tOutputEmpty && SIDES_VALID[mAutoOutput] && (mSuccessful || mInventoryChanged || !aReceivedEnoughEnergyBefore || mOutputBlocked == 1 || aTimer%1200 == 0)) {
			byte tAutoOutput = FACING_ROTATIONS[mFacing][mAutoOutput];
			DelegatorTileEntity<TileEntity> tTileEntity2 = getAdjacentTileEntity(tAutoOutput);
			for (int i = 0; i < mRecipes.mUsualOutputCount; i++) {
				UT.Inventories.moveOneItemStack(this, tTileEntity2, tAutoOutput, tTileEntity2.mSideOfTileEntity, null, F, (byte)64, (byte)1, (byte)64, (byte)1);
			}
		}
		
		tOutputEmpty = T;
		
		for (int i = mRecipes.mUsualInputCount, j = mRecipes.mUsualInputCount + mRecipes.mUsualOutputCount; i < j; i++) if (slot(i) != null) {tOutputEmpty = F; mOutputBlocked++; break;}
		
		if (tOutputEmpty) mOutputBlocked = 0;
		
    	if (mMaxProgress <= 0) {
    		if ((tRemovedOutputFluid || mSuccessful || mInventoryChanged || !aReceivedEnoughEnergyBefore || aTimer%1200 == 0) && checkRecipe() == FOUND_AND_SUCCESSFULLY_USED_RECIPE) {
		        onProcessStarted();
	    	} else {
	    		mProgress = 0;
	    	}
		}
		
		return rActive;
	}
	
	protected boolean doInactive(long aTimer, boolean aReceivedEnoughEnergyBefore) {
		mProgress = 0;
		return F;
	}
	
	public class MultiTileEntityGUICommonBasicMachine extends ContainerCommon {
		public MultiTileEntityGUICommonBasicMachine(InventoryPlayer aInventoryPlayer, MultiTileEntityBasicMachine aTileEntity) {
			super(aInventoryPlayer, aTileEntity);
		}
		
	    @Override
		public int addSlots(InventoryPlayer aPlayerInventory) {/*
	        addSlotToContainer(new Slot_Holo(mTileEntity, 0,  8, 63, false, true, 1));
	        addSlotToContainer(new Slot_Holo(mTileEntity, 0, 26, 63, false, true, 1));
	        addSlotToContainer(new Slot_Render(mTileEntity,  2, 107,  63));*/
	        
	        int tStartIndex = 0;
	        
	        switch (((MultiTileEntityBasicMachine)mTileEntity).mRecipes.mUsualInputCount) {
	        case  0:
	        	break;
	        case  1:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 25));
	        	break;
	        case  2:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 25));
	        	break;
	        case  3:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 25));
	        	break;
	        case  4:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 34));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 34));
	        	break;
	        case  5:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 34));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 34));
	        	break;
	        case  6:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 16));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 34));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 34));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 34));
	        	break;
	        case  7:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 43));
	        	break;
	        case  8:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 43));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 43));
	        	break;
	        default:
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53,  7));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 25));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 17, 43));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 35, 43));
	            addSlotToContainer(new Slot(mTileEntity, tStartIndex++, 53, 43));
	        	break;
	        }
	        
	        switch (((MultiTileEntityBasicMachine)mTileEntity).mRecipes.mUsualOutputCount) {
	        case  0:
	        	break;
	        case  1:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 25));
	        	break;
	        case  2:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 25));
	        	break;
	        case  3:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 25));
	        	break;
	        case  4:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 34));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 34));
	        	break;
	        case  5:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 34));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 34));
	        	break;
	        case  6:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 16));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 34));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 34));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 34));
	        	break;
	        case  7:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 43));
	        	break;
	        case  8:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 43));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 43));
	        	break;
	        default:
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143,  7));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 25));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 107, 43));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 125, 43));
	            addSlotToContainer(new Slot_Output(mTileEntity, tStartIndex++, 143, 43));
	        	break;
	        }
	        /*
	        addSlotToContainer(new Slot(mTileEntity,  1,  80,  63));
	        addSlotToContainer(new Slot(mTileEntity,  3, 125,  63));
	        addSlotToContainer(new Slot_Render(mTileEntity, tStartIndex++,  53,  63));*/
	        
	        return super.addSlots(aPlayerInventory);
	    }
	    
	    public short mProgressBar = 0;
	    
	    @Override
		public void detectAndSendChanges() {
	        super.detectAndSendChanges();
	    	for (ICrafting tUpdate : (List<ICrafting>)crafters) {
	    		if (((MultiTileEntityBasicMachine)mTileEntity).mSuccessful) {
	    			tUpdate.sendProgressBarUpdate(this, 0, Short.MAX_VALUE);
	    		} else if (((MultiTileEntityBasicMachine)mTileEntity).mMaxProgress > 0) {
	    			tUpdate.sendProgressBarUpdate(this, 0, (short)UT.Code.units(Math.min(((MultiTileEntityBasicMachine)mTileEntity).mMaxProgress, ((MultiTileEntityBasicMachine)mTileEntity).mProgress), ((MultiTileEntityBasicMachine)mTileEntity).mMaxProgress, Short.MAX_VALUE, T));
	    		} else {
	    			tUpdate.sendProgressBarUpdate(this, 0, -1);
	    		}
	    	}
	    }
	    
	    @Override
		@SideOnly(Side.CLIENT)
	    public void updateProgressBar(int aIndex, int aValue) {
	    	super.updateProgressBar(aIndex, aValue);
	    	switch (aIndex) {
	    	case 0: mProgressBar = (short)aValue; break;
	    	}
	    }
	    
	    @Override public int getSlotStartIndex() {return 0;}
	    @Override public int getShiftClickStartIndex() {return 0;}
	    @Override public int getSlotCount() {return ((MultiTileEntityBasicMachine)mTileEntity).mRecipes.mUsualInputCount + ((MultiTileEntityBasicMachine)mTileEntity).mRecipes.mUsualOutputCount;}
	    @Override public int getShiftClickSlotCount() {return ((MultiTileEntityBasicMachine)mTileEntity).mRecipes.mUsualInputCount;}
	}
	
	@SideOnly(Side.CLIENT)
	public class MultiTileEntityGUIClientBasicMachine extends ContainerClient {
		public final MultiTileEntityBasicMachine mTileEntity;
		
	    public MultiTileEntityGUIClientBasicMachine(InventoryPlayer aInventoryPlayer, MultiTileEntityBasicMachine aTileEntity) {
	        super(new MultiTileEntityGUICommonBasicMachine(aInventoryPlayer, aTileEntity), aTileEntity.mGUITexture);
	        mTileEntity = aTileEntity;
	        mNEI = mTileEntity.mRecipes.mNameNEI;
	    }
	    
	    @Override
	    protected void drawGuiContainerForegroundLayer(int par1, int par2) {
	        fontRendererObj.drawString(LH.get(mTileEntity.mRecipes.mNameInternal), 8,  4, 4210752);
	    }
	    
	    @Override
	    protected void drawGuiContainerBackgroundLayer(float par1, int par2, int par3) {
	    	super.drawGuiContainerBackgroundLayer(par1, par2, par3);
	        int x = (width - xSize) / 2;
	        int y = (height - ySize) / 2;
	        drawTexturedModalRect(x, y, 0, 0, xSize, ySize);
	        if (mContainer instanceof MultiTileEntityGUICommonBasicMachine) {
	        	if (((MultiTileEntityGUICommonBasicMachine)mContainer).mProgressBar >= 0) {
		        	int tSize = (mTileEntity.mRecipes.mProgressBarDirection % 4 < 2 ? 20 : 18), tProgress = (int)UT.Code.scale(((MultiTileEntityGUICommonBasicMachine)mContainer).mProgressBar, 0, Short.MAX_VALUE, tSize * UT.Code.unsignB(mTileEntity.mRecipes.mProgressBarAmount), F) % (tSize+1);
		        	
		        	switch (mTileEntity.mRecipes.mProgressBarDirection) {
		        	case 0:								drawTexturedModalRect(x + 78					, y + 24					, 176					, 0					, tProgress	, 18		); break;
		        	case 1:								drawTexturedModalRect(x + 78 + 20 - tProgress	, y + 24					, 176 + 20 - tProgress	, 0					, tProgress	, 18		); break;
		        	case 2:								drawTexturedModalRect(x + 78					, y + 24					, 176					, 0					, 20		, tProgress	); break;
		        	case 3:								drawTexturedModalRect(x + 78					, y + 24 + 18 - tProgress	, 176					, 18 - tProgress	, 20		, tProgress	); break;
		        	case 4: tProgress = 20 - tProgress;	drawTexturedModalRect(x + 78					, y + 24					, 176					, 0					, tProgress	, 18		); break;
		        	case 5: tProgress = 20 - tProgress;	drawTexturedModalRect(x + 78 + 20 - tProgress	, y + 24					, 176 + 20 - tProgress	, 0					, tProgress	, 18		); break;
		        	case 6: tProgress = 18 - tProgress;	drawTexturedModalRect(x + 78					, y + 24					, 176					, 0					, 20		, tProgress	); break;
		        	case 7: tProgress = 18 - tProgress;	drawTexturedModalRect(x + 78					, y + 24 + 18 - tProgress	, 176					, 18 - tProgress	, 20		, tProgress	); break;
		        	}
	        	}
	        }
	    }
	}
	
	protected void onProcessStarted() {/**/}
	protected void onProcessFinished() {/**/}
	
	@Override public Object getGUIClient2(int aGUIID, EntityPlayer aPlayer) {return new MultiTileEntityGUIClientBasicMachine(aPlayer.inventory, this);}
	@Override public Object getGUIServer2(int aGUIID, EntityPlayer aPlayer) {return new MultiTileEntityGUICommonBasicMachine(aPlayer.inventory, this);}
	
	@Override public void onFacingChange() {assignSideArrays();}
	@Override public byte getVisualData() {return (byte)((mActive?1:0)|(mRunning?2:0));}
	@Override public byte getDefaultSide() {return SIDE_SOUTH;}
	@Override public boolean[] getValidSides() {return SIDES_HORIZONTAL;}
	@Override public ITexture getTexture2(Block aBlock, int aRenderPass, byte aSide) {return new BlockTextureMulti(new BlockTextureDefault(mTexturesMaterial[FACING_ROTATIONS[mFacing][aSide]], mRGBa), new BlockTextureDefault((mActive||worldObj==null?mTexturesActive:mRunning?mTexturesRunning:mTexturesInactive)[FACING_ROTATIONS[mFacing][aSide]]));}
	
	@Override public String getTileEntityName() {return "gt.multitileentity.machine.basic";}
}